# OpenRouter Provider for Drupal AI

## INTRODUCTION

The OpenRouter Provider module integrates the [OpenRouter](https://openrouter.ai) API as a pluggable provider for the [Drupal AI framework](https://www.drupal.org/project/ai). It enables Drupal sites to leverage OpenRouter's large language models for chat completions and embeddings, using a secure and extensible architecture.

**Primary use cases:**
- Add OpenRouter as an AI provider for content generation, chatbots, and smart search
- Use OpenRouter's models for text embeddings (semantic search, recommendations, etc.)
- Provide a drop-in alternative to OpenAI and Anthropic providers, supporting flexible model selection
- Filter and curate available AI models to simplify selection throughout your site

## REQUIREMENTS

- Drupal 10 or 11
- [AI module](https://www.drupal.org/project/ai) (machine name `ai`)
- [Key module](https://www.drupal.org/project/key`) for secure API key storage
- PHP 8.1+
- Composer dependency: `openai-php/client`

## INSTALLATION

1. Install via Composer (from your project root):
   ```bash
   composer require openai-php/client
   ```
2. Enable the module:
   - Via the Drupal admin UI, or
   - With Drush: `drush en ai_provider_openrouter`
3. Ensure the required modules (`ai`, `key`) are also enabled.

## CONFIGURATION

1. **API Key:**
   - Go to Configuration → Web services → OpenRouter Provider.
   - Select a Key module entry containing your OpenRouter API key. (You may need to create a new key at `/admin/config/system/keys`.)
2. **API Base URL:**
   - The default is `https://openrouter.ai/api/v1`. Change only if using a custom endpoint.
3. **Model Selection:**
   - Enable specific models you want to use on your site from the hundreds available through OpenRouter.
   - Models are organized by provider (OpenAI, Anthropic, Claude, etc.) for easy selection.
   - If you enable specific models, only those will appear in dropdown selectors throughout your site.
   - If no models are enabled, all available models will be shown (default behavior).
4. **Provider Settings:**
   - The module registers as an AI provider and can be selected/configured via the AI module’s UI.

## MODEL FILTERING

The OpenRouter Provider module allows you to filter and curate the available AI models, making it easier to manage and select models throughout your site. This feature enables you to:

* Enable or disable specific models
* Organize models by provider for easy selection
* Restrict model selection to only enabled models (when at least one model is enabled)
* Show all available models when none are specifically enabled

This feature is particularly useful for large sites with multiple users, as it allows administrators to control which models are available for use. By default, if no models are explicitly enabled, all models will be available to ensure maximum flexibility.

## USAGE

- After configuration, OpenRouter will be available as a provider for AI-powered features (chat, embeddings, etc.) in the AI module.
- You can select models, adjust settings, and use OpenRouter for custom or contributed AI integrations.

## TROUBLESHOOTING

- **API Key Issues:**
  - Ensure your API key is valid and stored securely in the Key module.
  - Check permissions on the key entry.
- **Provider Not Available:**
  - Make sure the AI module and this provider are enabled.
  - Clear caches (`drush cr`) after installation/config changes.
- **Errors or Unexpected Results:**
  - Check logs at `/admin/reports/dblog` for details.
  - Review the OpenRouter API documentation for rate limits and model support.

## ROADMAP / TODO
- Implement additional AI operations (moderation, images, audio, etc.) as supported by OpenRouter.
- Provide Drush commands for admin/testing.
- Expand test coverage (unit, kernel tests).
- Complete streaming response implementation (UI setting exists but backend implementation is needed).

## CREDITS & MAINTAINERS

Current maintainers:
- Jeff Bruton (bdsweb) - https://www.drupal.org/u/bdsweb

Inspired by/contributions from:
- The Drupal AI module maintainers
- The OpenAI and Anthropic provider modules

## RESOURCES
- [OpenRouter API Docs](https://openrouter.ai/docs/api-reference/overview)
- [Drupal AI Initiative](https://www.drupal.org/project/artificial_intelligence_initiative)
- [Key module documentation](https://www.drupal.org/project/key)
